@@ -45,6 +45,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <limits.h>
 #include <sys/mman.h>
 #include <dirent.h>
 #include <errno.h>
@@ -122,7 +123,7 @@ pci_system_linux_sysfs_create( void )
  * \param d  Directory entry being processed by \c scandir.
  *
  * \return
- * Zero if the entry name matches either "." or "..", non-zero otherwise.
+ * Zero if the entry name matches either "." or ".."
  *
  * \sa scandir, populate_entries
  */
@@ -134,6 +135,56 @@ scan_sys_pci_filter( const struct dirent * d )
 }
 
 
+static int
+parse_separate_sysfs_files(struct pci_device * dev)
+{
+    static const char *attrs[] = {
+      "vendor",
+      "device",
+      "class",
+      "revision",
+      "subsystem_vendor",
+      "subsystem_device",
+    };
+    char name[256];
+    char resource[512];
+    uint64_t data[6];
+    int fd;
+    int i;
+
+    for (i = 0; i < 6; i++) {
+	snprintf(name, 255, "%s/%04x:%02x:%02x.%1u/%s",
+		 SYS_BUS_PCI,
+		 dev->domain,
+		 dev->bus,
+		 dev->dev,
+		 dev->func,
+		 attrs[i]);
+
+	fd = open(name, O_RDONLY | O_CLOEXEC);
+	if (fd == -1) {
+	    return errno;
+	}
+
+	read(fd, resource, 512);
+	resource[511] = '\0';
+
+	close(fd);
+
+	data[i] = strtoull(resource, NULL, 16);
+    }
+
+    dev->vendor_id = data[0] & 0xffff;
+    dev->device_id = data[1] & 0xffff;
+    dev->device_class = data[2] & 0xffffff;
+    dev->revision = data[3] & 0xff;
+    dev->subvendor_id = data[4] & 0xffff;
+    dev->subdevice_id = data[5] & 0xffff;
+
+    return 0;
+}
+
+
 int
 populate_entries( struct pci_system * p )
 {
@@ -157,15 +208,28 @@ populate_entries( struct pci_system * p )
 			(struct pci_device_private *) &p->devices[i];
 
 
-		sscanf(devices[i]->d_name, "%04x:%02x:%02x.%1u",
+		sscanf(devices[i]->d_name, "%x:%02x:%02x.%1u",
 		       & dom, & bus, & dev, & func);
 
 		device->base.domain = dom;
+		/*
+		 * Applications compiled with older versions  do not expect
+		 * 32-bit domain numbers. To keep them working, we keep a 16-bit
+		 * version of the domain number at the previous location.
+		 */
+		if (dom > 0xffff)
+		     device->base.domain_16 = 0xffff;
+		else
+		     device->base.domain_16 = dom;
 		device->base.bus = bus;
 		device->base.dev = dev;
 		device->base.func = func;
 
 
+		err = parse_separate_sysfs_files(& device->base);
+		if (!err)
+		    continue;
+
 		err = pci_device_linux_sysfs_read(& device->base, config, 0,
 						  48, & bytes);
 		if ((bytes == 48) && !err) {
@@ -591,8 +655,8 @@ pci_device_linux_sysfs_map_range(struct pci_device *dev,
 	    /* FIXME: Should we report an error in this case?
 	     */
 	    fprintf(stderr, "error setting MTRR "
-		    "(base = 0x%08lx, size = 0x%08x, type = %u) %s (%d)\n",
-		    sentry.base, sentry.size, sentry.type,
+		    "(base = 0x%016" PRIx64 ", size = 0x%08x, type = %u) %s (%d)\n",
+		    (pciaddr_t)sentry.base, sentry.size, sentry.type,
 		    strerror(errno), errno);
 /*            err = errno;*/
 	}
@@ -666,8 +730,8 @@ pci_device_linux_sysfs_unmap_range(struct pci_device *dev,
 	    /* FIXME: Should we report an error in this case?
 	     */
 	    fprintf(stderr, "error setting MTRR "
-		    "(base = 0x%08lx, size = 0x%08x, type = %u) %s (%d)\n",
-		    sentry.base, sentry.size, sentry.type,
+		    "(base = 0x%016" PRIx64 ", size = 0x%08x, type = %u) %s (%d)\n",
+		    (pciaddr_t)sentry.base, sentry.size, sentry.type,
 		    strerror(errno), errno);
 /*            err = errno;*/
 	}
