@@ -16,15 +16,15 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <sys/param.h>
 #include <sys/ioctl.h>
 #include <sys/mman.h>
 #include <sys/types.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #ifdef HAVE_MTRR
 #include <machine/sysarch.h>
 #include <machine/mtrr.h>
@@ -959,6 +959,10 @@ pci_system_netbsd_create(void)
 					continue;
 
 				device->base.domain = domain;
+				if (domain > 0xffff)
+				    device->base.domain_16 = 0xffff;
+				else
+				    device->base.domain_16 = domain & 0xffff;
 				device->base.bus = bus;
 				device->base.dev = dev;
 				device->base.func = func;
